BEGIN TRANSACTION;
CREATE TABLE objects_list(
	obj_id INTEGER PRIMARY KEY AUTOINCREMENT,
	obj_path TEXT NOT NULL UNIQUE,
	obj_hash TEXT NOT NULL,
	obj_type TEXT NOT NULL CHECK(obj_type in ("program","library","kmodule","other")),
	protected INTEGER NOT NULL DEFAULT 0 CHECK(protected in (0,1)),
	whlisted INTEGER NOT NULL DEFAULT 0 CHECK(whlisted in (0,1))
);

CREATE TRIGGER update_objects AFTER UPDATE OF whlisted,protected
ON objects_list
BEGIN
   DELETE FROM objects_list where obj_id = new.obj_id and whlisted = 0 and protected = 0;
END;

CREATE TRIGGER insert_objects BEFORE INSERT
ON objects_list WHEN new.protected = 0 and new.whlisted = 0
BEGIN
   SELECT RAISE(FAIL, "constraint check failed");
END;

CREATE VIEW whlisted_files AS
SELECT obj_id, obj_path, obj_hash, obj_type from objects_list where whlisted = 1;
CREATE VIEW protected_files AS
SELECT obj_id, obj_path, obj_hash, obj_type from objects_list where protected = 1;
COMMIT;

CREATE TRIGGER protected_delete
INSTEAD OF DELETE ON protected_files
BEGIN
	UPDATE objects_list SET protected = 0 WHERE obj_path=OLD.obj_path;
END;

CREATE TRIGGER protected_insert
INSTEAD OF INSERT ON protected_files WHEN EXISTS(select 1 from objects_list where obj_path=NEW.obj_path)
BEGIN
	UPDATE objects_list SET protected = 1 WHERE obj_path=NEW.obj_path;
END;

CREATE TRIGGER protected_update
INSTEAD OF INSERT ON protected_files WHEN NOT EXISTS(select 1 from objects_list where obj_path=NEW.obj_path)
BEGIN
	INSERT INTO objects_list(obj_path,obj_hash,obj_type,protected) values(NEW.obj_path, NEW.obj_hash, NEW.obj_type,1);
END;


CREATE TRIGGER whlisted_update
INSTEAD OF INSERT ON whlisted_files WHEN NOT EXISTS(select 1 from objects_list where obj_path=NEW.obj_path)
BEGIN
	INSERT INTO objects_list(obj_path,obj_hash,obj_type,whlisted) values(NEW.obj_path, NEW.obj_hash, NEW.obj_type,1);
END;

CREATE TRIGGER whlisted_insert
INSTEAD OF INSERT ON whlisted_files WHEN EXISTS(select 1 from objects_list where obj_path=NEW.obj_path)
BEGIN
	UPDATE objects_list SET whlisted = 1, obj_hash=NEW.obj_hash  WHERE obj_path=NEW.obj_path;
END;

CREATE TRIGGER whlisted_delete
INSTEAD OF DELETE ON whlisted_files
BEGIN
	UPDATE objects_list SET whlisted = 0 WHERE obj_path=OLD.obj_path;
END;
